//
//config.h
//
//functions related to turn configuration files and command lines into configuration.
//
//-UserX 2001/11/02
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include "isproxy.h"
#include "global.h"

#include "memiip.h"
#include "logger.h"
#include "config.h"
#include "parse.h"
#include "array.h"
#include "strarray.h"
#include "help.h"

StringIntPair boollist[] = {
		{"yes", 1},
		{"y", 1},
		{"true", 1},
		{"t", 1},
		{"on", 1},
		{"1", 1},

		{"no", 0},
		{"n", 0},
		{"false", 0},
		{"f", 0},
		{"off", 0},
		{"0", 0},

		{NULL, -1}
	};

char defaultconfigfilename[] = DEFAULT_CONFIGNAME;
char *configfilename = NULL;

ConfigEntry blankconfigentry = BLANKCONFIGENTRY;

ConfigEntryArrayHandle *cearray = NULL;

StringArrayHandle *cmdline = NULL;


void configInit(void) {
	if(cearray == NULL) {
		cearray = (ConfigEntryArrayHandle *) arrayMake(sizeof(ConfigEntry), 0, &blankconfigentry, NULL, NULL);
	}
}

//adds an array of config entries
//The array must be terminated with a 
//BLANKCONFIGENTRY (or CETYPE_NONE).
void configAddList(ConfigEntry *cea) {
	int i;
	int j;
	configInit();
	if(cea == NULL) {
		return;
	}
	for(i = 0; cea[i].type != CETYPE_NONE; i++) {
		j = cearray->size;
		arrayInsert((ArrayHandle *)cearray, j, 1);
		cearray->data[j] = cea[i];
	}
}

//reset the list of available config parameters
void configResetList(void) {
	configInit();
	arrayDelete((ArrayHandle *)cearray, 0, cearray->size);
}

int configMatchEntry(char *s) {
	int j;
	for(j = 0; j < cearray->size; j++) {
		if(stringCaseCompare(cearray->data[j].name.name, s) == 0) {
			return j;
		}
	}
	return -1;
}

int configValidate(ConfigEntryValidateFunc func, void *extra, void *value, char **s) {
	int result = 0;
	if(func != NULL) {
		func(extra, &result, value, s);
	}
	return result;
}

void configReadFile(char *config) {
	int i;
	int j;
	int v;
	char *s;
	char *ss;
	StringPairArrayHandle *spa;

	if(isStringBlank(config)) {
		return;
	}
	spa = spaReadFile(config);
	for(i = 0; i < spa->size; i++) {
		j = configMatchEntry(spa->data[i].key);
		if(j != -1) {
			switch(abs(cearray->data[j].type)) {
			case CETYPE_INT: //todo: format checking
				v = atoi(spa->data[i].val);
				if(configValidate(cearray->data[j].validatefunc, cearray->data[j].funcextra, (void *) &v, &ss) == 0){
					*cearray->data[j].i.data = v;
				} else {
					logmsg(LOG_ERROR, stringCopyMany(
							"Bad value: Key \"", 
							spa->data[i].key,
							"\" , value: \"",
							spa->data[i].val,
							"\": ", ss,
							NULL));
					stringFree(ss);
					ss = NULL;
				}
				break;
			case CETYPE_BOOL:
				switch(stringIntPairSearch(boollist, spa->data[i].val)) {
				case 1:
					*cearray->data[j].i.data = 1;
					break;
				case 0:
					*cearray->data[j].i.data = 0;
					break;
				case -1:
					logmsg(LOG_ERROR, stringCopyMany(
							"Bad value: key \"",
							spa->data[i].key,
							"\", value \"",
							spa->data[i].val,
							"\": Bad boolean value use 'true' or 'false'",
							NULL));
					break;
				}
				break;
			case CETYPE_STRING:
				stringFree(*cearray->data[j].s.data);
				s = stringCopy(spa->data[i].val);
				if(configValidate(cearray->data[j].validatefunc, cearray->data[j].funcextra, (void *)s, &ss) == 0) {
					*cearray->data[j].s.data = s;
				} else {
					logmsg(LOG_ERROR, stringCopyMany(
							"Bad value: Key \"", 
							spa->data[i].key,
							"\" , value: \"",
							spa->data[i].val,
							"\": ", ss,
							NULL));
					stringFree(ss);
					ss = NULL;
				}
				break;
			case CETYPE_FUNC:
				s = stringCopy(spa->data[i].val);
				if(configValidate(cearray->data[j].validatefunc, cearray->data[j].funcextra, (void *)s, &ss) == 0) {
					(cearray->data[j].getset.setfunc)(cearray->data[j].funcextra, s);
				} else {
					logmsg(LOG_ERROR, stringCopyMany(
							"Bad value: Key \"", 
							spa->data[i].key,
							"\" , value: \"",
							spa->data[i].val,
							"\": ", ss,
							NULL));
					stringFree(ss);
					ss = NULL;
				}
				break;
			default:
				logmsg(LOG_ERROR, stringCopyMany(
						"Internal error: Key \"", 
						spa->data[i].key,
						"\" , value: \"",
						spa->data[i].val,
						"\": unrecognized config entry type.",
						NULL));
			}
		} else {
			logmsg(LOG_ERROR, stringCopyMany(
					//"File: \"", 
					//filename,
					//"\", 
					"Configuration file: unknown key \"",
					spa->data[i].key,
					"\", with value \"",
					spa->data[i].val,
					"\"",
					NULL));
			break;
		}
	}
}

void configRead(void) {
	if(configfilename == NULL) {
		configReadFile(defaultconfigfilename);
	} else {
		configReadFile(configfilename);
	}
}

void configWriteFile(char *config) {
	FILE *fh;
	int i;
	int j;
	StringArrayHandle *sa;
	configInit();
	//todo:
	fh = fopen(config, "w");
	if(fh == NULL) {
		//todo: error message
		return;
	}
	for(i = 0; i < cearray->size; i++) {
		if(cearray->data[i].name.name != NULL) {
			switch(cearray->data[i].type) {
			case CETYPE_BOOL:
				fprintf(fh, "%s = %s\n", cearray->data[i].name.name, (*cearray->data[i].i.data)?"true":"false");
				break;
			case CETYPE_INT:
				fprintf(fh, "%s = %d\n", cearray->data[i].name.name, (*cearray->data[i].i.data));
				break;
			case CETYPE_STRING:
				if((*cearray->data[i].s.data) != NULL) {
					fprintf(fh, "%s = %s\n", cearray->data[i].name.name, (*cearray->data[i].s.data));
				} else {
					fprintf(fh, "%s = \n", cearray->data[i].name.name);
				}
				break;
			case CETYPE_FUNC:
				(cearray->data[i].getset.getfunc)(cearray->data[i].funcextra, &sa);
				for(j = 0; j < sa->size; j++) {
					fprintf(fh, "%s = %s\n", cearray->data[i].name.name, sa->data[j]);
				}
				saFree(sa);
				break;
			}
		}
	}
	fclose(fh);
}

void configWrite(void) {
	if(configfilename == NULL) {
		configWriteFile(defaultconfigfilename);
	} else {
		configWriteFile(configfilename);
	}
}
//resets all configuration options to defaults.
void configReset(void) {
	int i;
	configInit();

	for(i = 0; i < cearray->size; i++) {
		switch(cearray->data[i].type) {
		case CETYPE_BOOL:
		case CETYPE_INT:
			*cearray->data[i].i.data = cearray->data[i].i.datadefault;
			break;
		case CETYPE_STRING:
			stringFree(*cearray->data[i].s.data);
			*cearray->data[i].s.data = stringCopy(cearray->data[i].s.datadefault);
			break;
		case CETYPE_FUNC:
			(cearray->data[i].getset.resetfunc)(cearray->data[i].funcextra);
			break;
		}
	}
}

//Sets the name of the configuration file. or reset to the default if filename is blank or NULL.
void setConfigFile(char *filename) {
	
	stringFree(configfilename);
	if(filename == NULL || strlen(filename) == 0) {
		configfilename = NULL;
	} else {
		configfilename = stringCopy(filename);
	}
}

//read and parse command line parameters.
//parameters are indentical to that of main().
//void readCL(int argc, char **argv) {
void configReadCL() {
	int argc;
	char **argv;
	int p;

	if(cmdline == NULL) {
		return;
	}
	argc = cmdline->size;
	argv = cmdline->data;


// Parse the options
	for(p=1;p<argc;p++)
	{
		// a valid parameter
		if(argv[p][0]=='-')
		{
			// here for more advanced procesisng later...
			switch(argv[p][1])
			{
				case 'h':
				case 'H':
					showHelp();
					exit(0);
				case 'v':
					if(argv[p][2]==0) {
						setVerbosity(LOG_MINOR);
					} else {
						setVerbosity(atoi(&argv[p][2])); 
					}
					break;
				case 'd':
					doFork=FALSE;
					break;
				case 'p':
					localPort=atoi(&argv[p][2]);
					break;
				case 'l':
					setLogFileName(&argv[p][2]);
					break;
				case 'c':
					setConfigFile(&argv[p][2]);
					break;
				case 'S': //deleiberate bad memory release test
					memFree((void *) -1);//xifree((void *) -1);
					break;
				case 'K':
					makeKeys = -1;
					break;
				case 'C':
					doconfig = 1;
					break;
				default:
					printf("Invalid Parameter - type -h to see what is valid\n"
							"Sorry it didn't work out!\n");
					exit(1);
					break;
			}
		}
		// a number on it's own is a host to seed from (or |file) - this
		// won't work, of course, until there is seeding code written later
		else
		{
			printf("I'm sorry, the code for seeding hasn't been written yet\n"
					"Sorry it didn't work out!\n");
			exit(1);
		}
	}
}

void configSetCL(int argc, char **argv) {
	int i;
	saFree(cmdline);
	cmdline = saMake();
	for(i = 0; i < argc; i++) {
		saAppendCopy(cmdline, argv[i]);
	}
}

#if defined(__WIN32__) || defined(WIN32) || defined(_WIN32)


//reads windows command line options
//converts a string into  int argc, char **argv  and then calls readCL with them
//treats all whitespaces as sperators between comments
//void readWinCL(LPSTR lpcmdline) {
void configSetWinCL(LPSTR lpcmdline) {
	int i;
	int l;
	int ontext = FALSE;
	char *argbuffer = NULL;
	char **argv = NULL;
	int argc = 0;

	if (lpcmdline == NULL) return;

	argbuffer = stringAppend2("iip ", lpcmdline); //todo: change "iip " to what the actually program name is.
	l = strlen(argbuffer);
	for (i = 0; i < l + 1; i++) { 
		switch(argbuffer[i]) {
		case 0: //null terminator
		case ' ': //space
		case '\t': //tab
			argbuffer[i] = 0;
			if(ontext) {
				ontext = FALSE;
			}
			break;
		default:
			if(!ontext) {
				argc++;
				argv = memResize(argv, argc * sizeof(char *));//xirealloc(argv, argc * sizeof(char *));
				ontext = TRUE;
				argv[argc - 1] = argbuffer + i;
			}
		}
	}
	
	//readCL(argc, argv);
	configSetCL(argc, argv);

	memFree(argv);
	stringFree(argbuffer);

}
  
#endif //WIN32

