#ifndef CONFIG_H
#define CONFIG_H

#include "strarray.h"

typedef void (*ConfigEntrySetFunc)(void *, char *);
typedef void (*ConfigEntryGetFunc)(void *, StringArrayHandle **);
typedef void (*ConfigEntryResetFunc)(void *);
typedef void (*ConfigEntryValidateFunc)(void *, int *, void *, char **); //returns a non-zero int on a bad data format. descriptive message returned at the string pointer


typedef struct ConfigEntryName {
	char *name;
	char *clname;
} ConfigEntryName;

typedef struct ConfigEntryInt {
	int *data;
	int datadefault;
} ConfigEntryInt;
#define CEIB {NULL, 0}

typedef struct ConfigEntryString {
	char **data;
	char *datadefault;
} ConfigEntryString;
#define CESB {NULL, NULL}

typedef struct ConfigEntryFunc {
	ConfigEntrySetFunc setfunc;
	ConfigEntryGetFunc getfunc;
	ConfigEntryResetFunc resetfunc;
} ConfigEntryFunc;
#define CEFB {NULL, NULL, NULL}

typedef struct ConfigEntry {
	int type;
	ConfigEntryName name;
	ConfigEntryInt i;
	ConfigEntryString s;
	ConfigEntryFunc getset;
	ConfigEntryValidateFunc validatefunc;
	void *funcextra;
} ConfigEntry;

typedef struct ConfigEntryArrayHandle {
	int size;
	ConfigEntry *data;
} ConfigEntryArrayHandle;

#define DEFAULT_CONFIGNAME "isproxy.ini"

#define BLANKCONFIGENTRY {CETYPE_NONE,  {NULL, NULL}, CEIB, CESB, CEFB, NULL, NULL}

#define CETYPE_NONE 0
#define CETYPE_INT 1
#define CETYPE_BOOL 2
#define CETYPE_STRING 3
#define CETYPE_FUNC 4

#define CEVALIDATE_OK 1 //validate found the value to be ok
#define CEVALIDATE_BAD 2 //validate found the value to be bad
#define CEVALIDATE_FIXED 3 //validate adjusted the value to a correct one

void setConfigFile(char *filename);
//void readConfigFile(void);
void configSetCL(int argc, char **argv);
void configReadCL(void);
//void writeConfigFile(void);

#if defined(__WIN32__) || defined(WIN32) || defined(_WIN32)
	#include <windows.h>
	void configSetWinCL(LPSTR lpcmdline);
#endif


void configInit(void);
void configAddList(ConfigEntry *cea);
void configResetList(void);
int configMatchEntry(char *s);
int configValidate(ConfigEntryValidateFunc func, void *extra, void *value, char **s);
void configReadFile(char *config);
void configRead(void);
void configWriteFile(char *config);
void configWrite(void);
void configReset(void);

extern char *configfilename;

#endif //CONFIG_H
