//
//crypt-dh.c
//
//Code for making Diffie-Hellman keys
//
//
//-UserX 2001/11/15

#include "bignum.h"
#include "random.h"
#include "crypt-dh.h"

char Prime[] =
#ifdef DH_PUBLIC_KEY_LEN96
		"FFFFFFFF" "FFFFFFFF" "C90FDAA2" "2168C234" "C4C6628B" "80DC1CD1"
		"29024E08" "8A67CC74" "020BBEA6" "3B139B22" "514A0879" "8E3404DD"
		"EF9519B3" "CD3A431B" "302B0A6D" "F25F1437" "4FE1356D" "6D51C245"
		"E485B576" "625E7EC6" "F44C42E9" "A63A3620" "FFFFFFFF" "FFFFFFFF"
//todo: fix this
//#endif
//#ifdef DH_PUBLIC_KEY_LEN128
#else
		"FFFFFFFF" "FFFFFFFF" "C90FDAA2" "2168C234"
		"C4C6628B" "80DC1CD1" "29024E08" "8A67CC74"
		"020BBEA6" "3B139B22" "514A0879" "8E3404DD"
		"EF9519B3" "CD3A431B" "302B0A6D" "F25F1437"
		"4FE1356D" "6D51C245" "E485B576" "625E7EC6"
		"F44C42E9" "A637ED6B" "0BFF5CB6" "F406B7ED"
		"EE386BFB" "5A899FA5" "AE9F2411" "7C4B1FE6"
		"49286651" "ECE65381" "FFFFFFFF" "FFFFFFFF"
#endif
	;

//make a new private key
BigNum *dhGeneratePrivKey(void) {
	DataBlock *db = randomDBlock(DH_PRIVATE_KEY_LENGTH);
	BigNum *big = bignumFromBuffer(db->data, db->size);
	dblockFree(db);
	return big;
}

BigNum *dhGeneratePubKey(BigNum *PrivKey) {
	BigNum *result;
	BigNum *base = bignumFromInt(DH_BASE);
	BigNum *prime = bignumFromHex(Prime); //inefficient
	result = bignumExpModKeep(base, PrivKey, prime);
	bignumFree(prime);
	bignumFree(base);
	return result;
}

BigNum *dhGenerateSharedKey(BigNum *PrivKey, BigNum *RemotePubKey) {
	BigNum *result;
	BigNum *prime = bignumFromHex(Prime);
	result = bignumExpModKeep(RemotePubKey, PrivKey, prime);
	bignumFree(prime);
	return result;
}

