#ifndef CRYPT_H
#define CRYPT_H

#include "compat.h"
#include <stdlib.h>
#include "pipe.h"
#include "bignum.h"
#include "crypt-bf.h"

#define KEY_LENGTH 16 //XTEA_BLOCK_LENGTH
#define BLOCK_LENGTH 8 //XTEA_KEY_LENGTH
#define BLOCK_LENGTH1 7 //XTEA_BLOCK_LENGTH1;

typedef struct CryptStream{
	//Pipe *backPipe;
	Pipe *thisPipe;

	int inShake;
	int outShake;
	BigNum *privkey; //local private key
	BigNum *localkey; //local public key
	BigNum *remotekey; //remote public key
	BigNum *sharedkey; //the shared key
	DataBlock *insessionkey; //session key from key exchange
	DataBlock *outsessionkey; //session key from key exchange

	int inNeeded;
	int outCount;

	BlowfishContext *blowfishin;
	BlowfishContext *blowfishout;

	BigNum *setremotekey;

	uint8 counterin[BLOCK_LENGTH];
	uint8 counterout[BLOCK_LENGTH];

} CryptStream;

#define WRITES_TILL_ROTATE 52

#define BLANKCOUNTER {0, 0, 0, 0,  0, 0, 0, 0}
//#define BLANKCRYPTSTREAM {NULL, NULL, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, NULL, NULL, NULL, BLANKCOUNTER, BLANKCOUNTER}
#define BLANKCRYPTSTREAM {NULL, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, NULL, NULL, NULL, BLANKCOUNTER, BLANKCOUNTER}


CryptStream *csMake(void);
void csInit(CryptStream *cs, Pipe *thispipe);//, Pipe *backpipe
void csFree(CryptStream *cs);
void csDecrypt(CryptStream *cs);
void csEncrypt(CryptStream *cs);
void csInHandshake(CryptStream *cs);
void csOuthandshake(CryptStream *cs);
void csRead(CryptStream *cs);
void csWrite(CryptStream *cs);



//private functions
void csRotateSessionKey(DataBlock *key, DataBlock *rand);
void csApplyCounter(uint8 *data, uint8 *counter);
void csDecode(CryptStream *cs, DataBlock *db);
void csEncode(CryptStream *cs, DataBlock *db);
void csEncryptWrite(CryptStream *cs, DataBlock *db);
void csOutHandshake(CryptStream *cs);

#endif //CRYPT_H

