//
//pf-masq.c
//
//The pipeface for the 
//
//
//-UserX 2001/12/08

#include "pf-masq.h"
#include "pipe.h"
#include "pipeface.h"
#include "memiip.h"
#include "datablck.h"
#include "logger.h"
#include "striip.h"
#include "protomsq.h"

#include "p-dummy.h"

PipeFaceMasq blankpipefacemasq = BLANKPIPEFACEMASQ;

PipeFace *pfmasqMake(char *pipefacename, char *options) {
	PipeFaceMasq *pf = memCopy(&blankpipefacemasq, sizeof(blankpipefacemasq), "PipeFaceMasq", NULL);//xiclone(&blankpipefacemasq, sizeof(blankpipefacemasq));
	pipefaceMakeAt(&pf->pipeface);
	//pipefaceInitFuncs(pf, pfGetMsg, pfPutMsg, NULL);
	pipefaceInitFuncs((PipeFace *)pf, NULL, NULL, (PipeFaceFuncFree) pfmasqFree);
	pipeInitFunctions(&pf->pipeface.IOPipe, 
			(PipeFuncIn) pfmasqRead, (PipeFuncOut) pfmasqWrite, 
			NULL, NULL, 
			NULL, (PipeFuncStatus) pfmasqStatus);
	return (PipeFace *)pf;
}


void pfmasqFree(PipeFaceMasq *pf) {
	pipeClose(pf->masqpipe);
	memFree(pf);//xifree(pf);
}

void pfmasqDoIO(PipeFaceMasq *pf) {
	int v;
	pipeGenericRead(pf->masqpipe);
	pipeGenericWrite(pf->masqpipe);
	do {
		if(pf->inwaiting == 0) {
			if(pf->pipeface.IOPipe.backPipe->inBuffer->size >= 2) {
				pf->inwaiting = dblockReadInt(pf->pipeface.IOPipe.backPipe->inBuffer, 2);
			} else {
				break;
			}
		}
		if(pf->inwaiting != 0) {
			v = abs(pf->inwaiting);
			if(pf->pipeface.IOPipe.backPipe->inBuffer->size >= v) {
				if(pf->inwaiting > 0) {
					LOGDEBUGTRAFFIC(stringJoinMany(
							"pfmasqDoIO: ",
							ptrToString(pf),
							", from core bytes:",
							v,
						NULL));
					//traffic coming from the core
					pf->masqpipe->outBuffer = dblockMove(pf->masqpipe->outBuffer, pf->pipeface.IOPipe.backPipe->inBuffer, v);
				} else {
					LOGDEBUGTRAFFIC(stringJoinMany(
							"pfmasqDoIO: ",
							ptrToString(pf),
							", from socket bytes:",
							v,
						NULL));
					//traffic coming from the socket
					pf->masqpipe->backPipe->inBuffer = dblockMove(pf->masqpipe->backPipe->inBuffer, pf->pipeface.IOPipe.backPipe->inBuffer, v);
				}
				pf->inwaiting = 0;
			} else {
				break;
			}
		}
	} while(pf->inwaiting == 0);

	pipeGenericRead(pf->masqpipe);
	pipeGenericWrite(pf->masqpipe);

	while(pf->masqpipe->inBuffer->size > 0) {
		//traffic going towards the core
		v = pf->masqpipe->inBuffer->size;
		if(v > 0x7fff) {
			v = 0x7fff;
		}
		LOGDEBUGTRAFFIC(stringJoinMany(
				"pfmasqDoIO: ",
				ptrToString(pf),
				", to core bytes:",
				v,
			NULL));
		pf->pipeface.IOPipe.outBuffer = dblockWriteInt(pf->pipeface.IOPipe.outBuffer, v, 2);
		pf->pipeface.IOPipe.outBuffer = dblockMove(pf->pipeface.IOPipe.outBuffer, pf->masqpipe->inBuffer, v);
	}

	while(pf->masqpipe->backPipe->outBuffer->size > 0) {
		//traffic going towards the socket
		v = pf->masqpipe->backPipe->outBuffer->size;
		if(v > 0x7fff) {
			v = 0x7fff;
		}
		LOGDEBUGTRAFFIC(stringJoinMany(
				"pfmasqDoIO: ",
				ptrToString(pf),
				", to socket bytes:",
				v,
			NULL));
		pf->pipeface.IOPipe.outBuffer = dblockWriteInt(pf->pipeface.IOPipe.outBuffer, v, 2);
		pf->pipeface.IOPipe.outBuffer = dblockMove(pf->pipeface.IOPipe.outBuffer, pf->masqpipe->backPipe->outBuffer, v);
	}
}

void pfmasqRead(PipeFaceMasq *pf, Pipe **errPipe) {
	*errPipe = pipeGenericRead(pf->pipeface.IOPipe.backPipe);
	if(*errPipe == NULL) {
		//LOGDEBUGTRAFFIC(stringJoinMany(
		//		"pfmasqRead: ",
		//		ptrToString(pf),
		//		", read bytes:",
		//		intToHexString(thispipe->IOPipe.backPipe->inBuffer->size),
		//	NULL));
		//thispipe->IOPipe.inBuffer = dblockMoveAll(thispipe->IOPipe.inBuffer, thispipe->IOPipe.backPipe->inBuffer);
		if(pf->protocol == NULL) {
			pf->protocol = dblockGetString(pf->pipeface.IOPipe.backPipe->inBuffer);
			pf->masqpipe = protocolGetPipeMasq(pf->protocol, ""); //todo: add options
			if(pf->masqpipe == NULL) {
				stringFree(pf->protocol);
				pf->protocol = NULL;
				pf->pipeface.IOPipe.status |= PSTATUS_CLOSED;
			} else {
				pipeAttach(pipedummyMake("", ""), pf->masqpipe);
			}
		}
		if(pf->masqpipe != NULL) {
			pfmasqDoIO(pf);
		}
	}
}

void pfmasqWrite(PipeFaceMasq *pf, Pipe **errPipe) {
	//LOGDEBUGTRAFFIC(stringJoinMany(
	//		"pfmasqWrite: ",
	//		ptrToString(pf),
	//		", write bytes:",
	//		intToHexString(thispipe->IOPipe.outBuffer->size),
	//	NULL));
	//thispipe->IOPipe.backPipe->outBuffer = dblockMoveAll(thispipe->IOPipe.backPipe->outBuffer, thispipe->IOPipe.outBuffer);
	if(pf->masqpipe != NULL) {
		pfmasqDoIO(pf);
	}
	*errPipe = pipeGenericWrite(pf->pipeface.IOPipe.backPipe);
}


void pfmasqStatus(PipeFaceMasq *pf, int *status) {
	*status |= pipeStatus(pf->masqpipe);
	if((*status & PSTATUS_CLOSED) != 0) {
		pf->pipeface.IOPipe.status |= PSTATUS_CLOSED;
	}
}

