//
//striosck.c
//
//Functions relevant to doing string operations with files and sockets.
//
//
//-UserX 2001/11/04

#include "striosck.h"
#include "datablck.h"

//read a line of text from a file.
// carriage returns are discarded.
// linefeeds indicate end of line.
char *stringReadLine(FILE *fh){
	int c;
	DataBlock *db = dblockEmpty("stringReadLine");
	char *buffer;

	if(fh == NULL) {
		dblockFree(db);
		return NULL;
	}

	while(!feof(fh) && !ferror(fh)) {
		c = fgetc(fh);
		if(c == EOF) {
			break;
		}
		if(c == '\n') {
			break;
		}
		switch(c) {
		case 0:
		case '\r':
			break;
		default:
			db = dblockResize(db, db->size + 1);
			db->data[db->size] = (uint8) c;
			db->size++;
		}
	}
	db = dblockResize(db, db->size + 1);
	db->data[db->size] = 0;
	db->size++;

	buffer = stringCopy((char *)db->data);

	dblockFree(db);

	return buffer;
}

char *stringReadLineOrEOF(FILE *fh) {
	int c = 0;
	DataBlock *db = dblockEmpty("stringReadLine");
	char *buffer;

	if(fh == NULL) {
		dblockFree(db);
		return NULL;
	}

	while(!feof(fh) && !ferror(fh)) {
		c = fgetc(fh);
		if(c == EOF) {
			break;
		}
		if(c == '\n') {
			break;
		}
		switch(c) {
		case 0:
		case '\r':
			break;
		default:
			db = dblockResize(db, db->size + 1);
			db->data[db->size] = (uint8) c;
			db->size++;
		}
	}
	//if(c == EOF || feof(fh) || ferror(fh)) {
		if(db->size == 0) {
			db = dblockResize(db, db->size + 1);
			db->data[db->size] = '\x1a';
			db->size++;
		}
	//}
	db = dblockResize(db, db->size + 1);
	db->data[db->size] = 0;
	db->size++;

	buffer = stringCopy((char *)db->data);

	dblockFree(db);

	return buffer;
}

//write a string to a file.
//string will be released
void stringWriteLine(char *text, FILE *fh) {
	fprintf(fh, "%s\n", text);
}
